const express = require('express');
const cors = require('cors');
const fs = require('fs');
const { exec } = require('child_process');
const path = require('path');
const http = require('http'); // Required to create a separate server for port 80

const app = express();
const PORT = 3666;
const PORT80 = 80;

// Enable CORS
app.use(cors());

// Middleware to parse JSON bodies
app.use(express.json());

// GET endpoint to test connectivity
app.get('/test', (req, res) => {
    res.json({
        status: 'ok',
        timestamp: new Date().toISOString(),
    });
});

// POST endpoint to run commands
app.post('/run', (req, res) => {
    let { prompt, faas } = req.body;

    console.log('RAW FAAS', faas);

    let invalidJSON = false;
    try {
        faas = JSON.parse(faas);
    } catch (err) {
        invalidJSON = true;
    }

    if (invalidJSON) {
        try {
            faas = `[${faas}]`;
            faas = JSON.parse(faas);
        } catch (err) {
            return res.status(400).json({ error: 'Invalid JSON in faas' });
        }
    }

    console.log('GOT FAAS', faas);

    if (!faas) {
        return res.status(400).json({ error: 'Invalid faas format, variable typeof (' + typeof faas + ') given.' });
    }

    if (!Array.isArray(faas)) {
        faas = [faas];
    }

    const results = []; // To store results of each task
    let completed = 0; // Track completed tasks

    faas.forEach((task, index) => {
        // Validate input
        if (!task || typeof task !== 'object') {
            results[index] = { error: 'Invalid task format' };
            checkCompletion();
            return;
        }

        const { uuid, type, subtype, content, meta } = task;

        // Ensure required fields are present
        if (!uuid || !type || !content) {
            results[index] = { error: 'Missing required fields in task' };
            checkCompletion();
            return;
        }

        try {
            // Handle different command types
            if (type === 'command' && content) {
                // Execute bash command directly
                exec(content, { shell: '/bin/bash' }, (error, stdout, stderr) => {
                    if (error) {
                        results[index] = {
                            uuid,
                            success: false,
                            error: error.message,
                            stderr: stderr.trim(),
                        };
                    } else {
                        results[index] = {
                            uuid,
                            success: true,
                            stdout: stdout, // Preserve raw terminal-style output
                            stderr: stderr,
                        };
                    }
                    checkCompletion();
                });
            } else if (type === 'file' && subtype === 'save') {
                const { filepath } = meta;

                console.log(filepath);
                console.log(content);

                if (!filepath || !content) {
                    results[index] = { uuid, error: 'Missing filepath in meta or content for file save' };
                    checkCompletion();
                    return;
                }

                fs.writeFile(filepath, content, 'utf8', (err) => {
                    if (err) {
                        results[index] = { uuid, error: 'Failed to save file', details: err.message };
                    } else {
                        results[index] = { uuid, success: true, message: `File saved as ${filepath}` };
                    }
                    checkCompletion();
                });
            } else {
                results[index] = { uuid, error: 'Unsupported command type or subtype' };
                checkCompletion();
            }
        } catch (err) {
            results[index] = { uuid, error: 'Internal server error', details: err.message };
            checkCompletion();
        }
    });

    // Function to check if all tasks are complete
    function checkCompletion() {
        completed++;
        if (completed === faas.length) {
            res.json(results); // Send all results as a single response
        }
    }
});

// Start the Express server on port 3666
app.listen(PORT, () => {
    console.log(`Server running on http://localhost:${PORT}`);
});

const server80 = http.createServer((req, res) => {
    // Get the requested file path from the URL
    const requestedPath = req.url;
    const localPath = path.join(__dirname, requestedPath); // Join it to the current directory

    // Check if the requested file exists
    fs.exists(localPath, (exists) => {
        if (exists) {
            // Read the file and send it as the response
            fs.readFile(localPath, 'utf8', (err, data) => {
                if (err) {
                    res.statusCode = 500;
                    res.setHeader('Access-Control-Allow-Origin', '*'); // ✅ Allow requests from any origin
                    res.setHeader('Access-Control-Allow-Methods', 'GET, OPTIONS'); // ✅ Allow GET & OPTIONS
                    res.end('Internal server error');
                } else {
                    res.setHeader('Content-Type', 'text/plain');
                    res.setHeader('Access-Control-Allow-Origin', '*'); // ✅ Enable CORS
                    res.setHeader('Access-Control-Allow-Methods', 'GET, OPTIONS'); // ✅ Allow preflight
                    res.statusCode = 200;
                    res.end(data); // Send the file content
                }
            });
        } else {
            // If file doesn't exist, return 404
            res.statusCode = 404;
            res.setHeader('Access-Control-Allow-Origin', '*'); // ✅ Enable CORS for 404 responses too
            res.end('File not found');
        }
    });
});

// Start the server on port 80
server80.listen(PORT80, () => {
    console.log(`File server running on http://localhost:${PORT80}`);
});
