#!/usr/bin/env python3

import sys
from PIL import Image, ImageDraw, ImageFont

def measure_text(text, font):
    """
    Returns (width, height) of the given text when rendered with the given font,
    using a fallback approach for older PIL/Pillow versions that do not support
    font.getsize() or draw.textsize().
    """
    # If the text is empty or just whitespace, measure at least one space
    # so blank lines still have a detectable height.
    text_to_measure = text if text.strip() else " "
    # getmask(...) returns a pixel mask of the rendered text; .size is (width, height)
    mask = font.getmask(text_to_measure)
    return mask.size

def convert_text_to_png(input_path, output_path,
                        font_path=None,
                        font_size=20,
                        max_width=1200,
                        bg_color=(255, 255, 255),
                        text_color=(0, 0, 0),
                        margin=50,
                        line_spacing=5):

    # Read file as raw text
    with open(input_path, 'r', encoding='utf-8') as f:
        text = f.read()

    # Load font
    if font_path:
        font = ImageFont.truetype(font_path, font_size)
    else:
        font = ImageFont.load_default()

    # Wrap text lines to avoid exceeding max_width
    lines = []
    for original_line in text.split('\n'):
        words = original_line.split(' ')
        current_line = ""
        for word in words:
            test_line = (current_line + " " + word).strip()
            line_width, _ = measure_text(test_line, font)
            if line_width <= max_width:
                current_line = test_line
            else:
                lines.append(current_line)
                current_line = word
        lines.append(current_line)

    # Calculate image size
    max_line_width = 0
    total_height = 0

    for line in lines:
        w, h = measure_text(line, font)
        max_line_width = max(max_line_width, w)
        total_height += (h + line_spacing)

    img_width = max_line_width + (margin * 2)
    img_height = total_height + (margin * 2)

    # Create image & draw text
    img = Image.new('RGB', (img_width, img_height), color=bg_color)
    draw = ImageDraw.Draw(img)

    y_offset = margin
    for line in lines:
        draw.text((margin, y_offset), line, fill=text_color, font=font)
        _, h = measure_text(line, font)
        y_offset += (h + line_spacing)

    # Save to PNG
    img.save(output_path, 'PNG')

def main():
    if len(sys.argv) < 3:
        print("Usage: python text_to_png.py <input_file> <output_file> [<font_path>]")
        sys.exit(1)

    input_file = sys.argv[1]
    output_file = sys.argv[2]
    font_file = sys.argv[3] if len(sys.argv) > 3 else None

    convert_text_to_png(
        input_path=input_file,
        output_path=output_file,
        font_path=font_file,
        font_size=20,
        max_width=1200
    )

if __name__ == "__main__":
    main()
